@@ -22,6 +22,7 @@
 #include <QMimeDatabase>
 #include <QTimer>
 
+#include <KLazyLocalizedString>
 #include <KLocalizedString>
 #include <kfileitem.h>
 
@@ -138,18 +139,18 @@ static QUrl createSearchUrl(const QUrl &url)
 }
 }
 
-class Q_DECL_HIDDEN KFilePlacesModel::Private
+class KFilePlacesModelPrivate
 {
 public:
-    explicit Private(KFilePlacesModel *self)
-        : q(self)
+    explicit KFilePlacesModelPrivate(KFilePlacesModel *qq)
+        : q(qq)
         , bookmarkManager(nullptr)
         , fileIndexingEnabled(isFileIndexingEnabled())
         , tags()
         , tagsLister(new KCoreDirLister(q))
     {
         if (KProtocolInfo::isKnownProtocol(QStringLiteral("tags"))) {
-            connect(tagsLister, &KCoreDirLister::itemsAdded, q, [this](const QUrl &, const KFileItemList &items) {
+            QObject::connect(tagsLister, &KCoreDirLister::itemsAdded, q, [this](const QUrl &, const KFileItemList &items) {
                 if (tags.isEmpty()) {
                     QList<QUrl> existingBookmarks;
 
@@ -163,8 +164,7 @@ public:
 
                     if (!existingBookmarks.contains(QUrl(tagsUrlBase))) {
                         KBookmark alltags = KFilePlacesItem::createSystemBookmark(bookmarkManager,
-                                                                                  "All tags",
-                                                                                  i18n("All tags").toUtf8().data(),
+                                                                                  kli18nc("KFile System Bookmarks", "All tags").untranslatedText(),
                                                                                   QUrl(tagsUrlBase),
                                                                                   QStringLiteral("tag"));
                     }
@@ -177,25 +177,20 @@ public:
                         tags.append(name);
                     }
                 }
-                _k_reloadBookmarks();
+                reloadBookmarks();
             });
 
-            connect(tagsLister, &KCoreDirLister::itemsDeleted, q, [this](const KFileItemList &items) {
+            QObject::connect(tagsLister, &KCoreDirLister::itemsDeleted, q, [this](const KFileItemList &items) {
                 for (const KFileItem &item : items) {
                     tags.removeAll(item.name());
                 }
-                _k_reloadBookmarks();
+                reloadBookmarks();
             });
 
             tagsLister->openUrl(QUrl(tagsUrlBase), KCoreDirLister::OpenUrlFlag::Reload);
         }
     }
 
-    ~Private()
-    {
-        qDeleteAll(items);
-    }
-
     KFilePlacesModel *const q;
 
     QList<KFilePlacesItem *> items;
@@ -218,13 +213,13 @@ public:
     const QString tagsUrlBase = QStringLiteral("tags:/");
     KCoreDirLister *tagsLister;
 
-    void _k_initDeviceList();
-    void _k_deviceAdded(const QString &udi);
-    void _k_deviceRemoved(const QString &udi);
-    void _k_itemChanged(const QString &udi);
-    void _k_reloadBookmarks();
-    void _k_storageSetupDone(Solid::ErrorType error, const QVariant &errorData);
-    void _k_storageTeardownDone(Solid::ErrorType error, const QVariant &errorData);
+    void initDeviceList();
+    void deviceAdded(const QString &udi);
+    void deviceRemoved(const QString &udi);
+    void itemChanged(const QString &udi);
+    void reloadBookmarks();
+    void storageSetupDone(Solid::ErrorType error, const QVariant &errorData, Solid::StorageAccess *sender);
+    void storageTeardownDone(Solid::ErrorType error, const QVariant &errorData);
 
 private:
     bool isBalooUrl(const QUrl &url) const;
@@ -250,7 +245,7 @@ static inline QString versionKey()
 
 KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QObject *parent)
     : QAbstractItemModel(parent)
-    , d(new Private(this))
+    , d(new KFilePlacesModelPrivate(this))
 {
     const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/user-places.xbel");
     d->bookmarkManager = KBookmarkManager::managerForExternalFile(file);
@@ -276,60 +271,64 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
 
         /* clang-format off */
         auto createSystemBookmark =
-            [this, &seenUrls](const char *translationContext,
-                              const QByteArray &untranslatedLabel,
+            [this, &seenUrls](const char *untranslatedLabel,
                               const QUrl &url,
                               const QString &iconName,
                               const KBookmark &after) {
                 if (!seenUrls.contains(url)) {
-                    return KFilePlacesItem::createSystemBookmark(d->bookmarkManager, translationContext, untranslatedLabel, url, iconName, after);
+                    return KFilePlacesItem::createSystemBookmark(d->bookmarkManager, untranslatedLabel, url, iconName, after);
                 }
                 return KBookmark();
             };
         /* clang-format on */
 
         if (fileVersion < 2) {
-            // NOTE: The context for these I18NC_NOOP calls has to be "KFile System Bookmarks".
+            // NOTE: The context for these kli18nc calls has to be "KFile System Bookmarks".
             // The real i18nc call is made later, with this context, so the two must match.
-            // createSystemBookmark actually does nothing with its second argument, the context.
-            createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Home"), QUrl::fromLocalFile(QDir::homePath()), QStringLiteral("user-home"), KBookmark());
+            createSystemBookmark(kli18nc("KFile System Bookmarks", "Home").untranslatedText(),
+                                 QUrl::fromLocalFile(QDir::homePath()),
+                                 QStringLiteral("user-home"),
+                                 KBookmark());
 
             // Some distros may not create various standard XDG folders by default
             // so check for their existence before adding bookmarks for them
             const QString desktopFolder = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
             if (QDir(desktopFolder).exists()) {
-                createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Desktop"),
+                createSystemBookmark(kli18nc("KFile System Bookmarks", "Desktop").untranslatedText(),
                                      QUrl::fromLocalFile(desktopFolder),
                                      QStringLiteral("user-desktop"),
                                      KBookmark());
             }
             const QString documentsFolder = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
             if (QDir(documentsFolder).exists()) {
-                createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Documents"),
+                createSystemBookmark(kli18nc("KFile System Bookmarks", "Documents").untranslatedText(),
                                      QUrl::fromLocalFile(documentsFolder),
                                      QStringLiteral("folder-documents"),
                                      KBookmark());
             }
             const QString downloadFolder = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
             if (QDir(downloadFolder).exists()) {
-                createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Downloads"),
+                createSystemBookmark(kli18nc("KFile System Bookmarks", "Downloads").untranslatedText(),
                                      QUrl::fromLocalFile(downloadFolder),
                                      QStringLiteral("folder-downloads"),
                                      KBookmark());
             }
-            createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Network"),
+            createSystemBookmark(kli18nc("KFile System Bookmarks", "Network").untranslatedText(),
                                  QUrl(QStringLiteral("remote:/")),
                                  QStringLiteral("folder-network"),
                                  KBookmark());
 
-            createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Trash"), QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"), KBookmark());
+            createSystemBookmark(kli18nc("KFile System Bookmarks", "Trash").untranslatedText(),
+                                 QUrl(QStringLiteral("trash:/")),
+                                 QStringLiteral("user-trash"),
+                                 KBookmark());
         }
 
         if (!newFile && fileVersion < 3) {
-            KBookmarkGroup root = d->bookmarkManager->root();
-            KBookmark bItem = root.first();
+            KBookmarkGroup rootGroup = d->bookmarkManager->root();
+            KBookmark bItem = rootGroup.first();
             while (!bItem.isNull()) {
-                KBookmark nextbItem = root.next(bItem);
+                KBookmark nextbItem = rootGroup.next(bItem);
                 const bool isSystemItem = bItem.metaDataItem(QStringLiteral("isSystemItem")) == QLatin1String("true");
                 if (isSystemItem) {
                     const QString text = bItem.fullText();
@@ -348,11 +347,11 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
                     } else if (text == QLatin1String("This Month")) {
                         // Because of 7e1d2fb84546506c91684dd222c2485f0783848f we need to find the system bookmark named This Month
                         // and remove it, otherwise users are getting untranslated strings
-                        root.deleteBookmark(bItem);
+                        rootGroup.deleteBookmark(bItem);
                     } else if (text == QLatin1String("Last Month")) {
                         // Because of 7e1d2fb84546506c91684dd222c2485f0783848f we need to find the system bookmark named Last Month
                         // and remove it, otherwise users are getting untranslated strings
-                        root.deleteBookmark(bItem);
+                        rootGroup.deleteBookmark(bItem);
                     }
                 }
 
@@ -380,14 +379,14 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
 
             const QString musicFolder = QStandardPaths::writableLocation(QStandardPaths::MusicLocation);
             if (QDir(musicFolder).exists()) {
-                after = createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Music"),
+                after = createSystemBookmark(kli18nc("KFile System Bookmarks", "Music").untranslatedText(),
                                              QUrl::fromLocalFile(musicFolder),
                                              QStringLiteral("folder-music"),
                                              after);
             }
             const QString pictureFolder = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
             if (QDir(pictureFolder).exists()) {
-                after = createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Pictures"),
+                after = createSystemBookmark(kli18nc("KFile System Bookmarks", "Pictures").untranslatedText(),
                                              QUrl::fromLocalFile(pictureFolder),
                                              QStringLiteral("folder-pictures"),
                                              after);
@@ -396,7 +395,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
             // is called normally on Linux: https://cgit.freedesktop.org/xdg/xdg-user-dirs/tree/user-dirs.defaults
             const QString videoFolder = QStandardPaths::writableLocation(QStandardPaths::MoviesLocation);
             if (QDir(videoFolder).exists()) {
-                after = createSystemBookmark(I18NC_NOOP("KFile System Bookmarks", "Videos"),
+                after = createSystemBookmark(kli18nc("KFile System Bookmarks", "Videos").untranslatedText(),
                                              QUrl::fromLocalFile(videoFolder),
                                              QStringLiteral("folder-videos"),
                                              after);
@@ -423,12 +422,12 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         root.setMetaDataItem(QStringLiteral("withRecentlyUsed"), QStringLiteral("true"));
 
         KBookmark recentFilesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                                              I18NC_NOOP("KFile System Bookmarks", "Recent Files"),
+                                                                              kli18nc("KFile System Bookmarks", "Recent Files").untranslatedText(),
                                                                               QUrl(QStringLiteral("recentlyused:/files")),
                                                                               QStringLiteral("document-open-recent"));
 
         KBookmark recentDirectoriesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                                                    I18NC_NOOP("KFile System Bookmarks", "Recent Locations"),
+                                                                                    kli18nc("KFile System Bookmarks", "Recent Locations").untranslatedText(),
                                                                                     QUrl(QStringLiteral("recentlyused:/locations")),
                                                                                     QStringLiteral("folder-open-recent"));
 
@@ -451,29 +450,29 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         // don't add by default "Modified Today" and "Modified Yesterday" when recentlyused:/ is present
         if (root.metaDataItem(QStringLiteral("withRecentlyUsed")) != QLatin1String("true")) {
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                  I18NC_NOOP("KFile System Bookmarks", "Modified Today"),
+                                                  kli18nc("KFile System Bookmarks", "Modified Today").untranslatedText(),
                                                   QUrl(QStringLiteral("timeline:/today")),
                                                   QStringLiteral("go-jump-today"));
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                  I18NC_NOOP("KFile System Bookmarks", "Modified Yesterday"),
+                                                  kli18nc("KFile System Bookmarks", "Modified Yesterday").untranslatedText(),
                                                   QUrl(QStringLiteral("timeline:/yesterday")),
                                                   QStringLiteral("view-calendar-day"));
         }
 
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              I18NC_NOOP("KFile System Bookmarks", "Documents"),
+                                              kli18nc("KFile System Bookmarks", "Documents").untranslatedText(),
                                               QUrl(QStringLiteral("search:/documents")),
                                               QStringLiteral("folder-text"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              I18NC_NOOP("KFile System Bookmarks", "Images"),
+                                              kli18nc("KFile System Bookmarks", "Images").untranslatedText(),
                                               QUrl(QStringLiteral("search:/images")),
                                               QStringLiteral("folder-images"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              I18NC_NOOP("KFile System Bookmarks", "Audio"),
+                                              kli18nc("KFile System Bookmarks", "Audio").untranslatedText(),
                                               QUrl(QStringLiteral("search:/audio")),
                                               QStringLiteral("folder-sound"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              I18NC_NOOP("KFile System Bookmarks", "Videos"),
+                                              kli18nc("KFile System Bookmarks", "Videos").untranslatedText(),
                                               QUrl(QStringLiteral("search:/videos")),
                                               QStringLiteral("folder-videos"));
 
@@ -504,15 +503,15 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
     Q_ASSERT(d->predicate.isValid());
 
     connect(d->bookmarkManager, &KBookmarkManager::changed, this, [this]() {
-        d->_k_reloadBookmarks();
+        d->reloadBookmarks();
     });
     connect(d->bookmarkManager, &KBookmarkManager::bookmarksChanged, this, [this]() {
-        d->_k_reloadBookmarks();
+        d->reloadBookmarks();
     });
 
-    d->_k_reloadBookmarks();
+    d->reloadBookmarks();
     QTimer::singleShot(0, this, [this]() {
-        d->_k_initDeviceList();
+        d->initDeviceList();
     });
 }
 
@@ -521,10 +520,7 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
 {
 }
 
-KFilePlacesModel::~KFilePlacesModel()
-{
-    delete d;
-}
+KFilePlacesModel::~KFilePlacesModel() = default;
 
 QUrl KFilePlacesModel::url(const QModelIndex &index) const
 {
@@ -692,7 +688,7 @@ QModelIndex KFilePlacesModel::closestItem(const QUrl &url) const
     for (int row = 0; row < d->items.size(); ++row) {
         KFilePlacesItem *item = d->items[row];
 
-        if (item->isHidden()) {
+        if (item->isHidden() || isGroupHidden(item->groupType())) {
             continue;
         }
 
@@ -714,15 +710,15 @@ QModelIndex KFilePlacesModel::closestItem(const QUrl &url) const
     }
 }
 
-void KFilePlacesModel::Private::_k_initDeviceList()
+void KFilePlacesModelPrivate::initDeviceList()
 {
     Solid::DeviceNotifier *notifier = Solid::DeviceNotifier::instance();
 
-    connect(notifier, &Solid::DeviceNotifier::deviceAdded, q, [this](const QString &device) {
-        _k_deviceAdded(device);
+    QObject::connect(notifier, &Solid::DeviceNotifier::deviceAdded, q, [this](const QString &device) {
+        deviceAdded(device);
     });
-    connect(notifier, &Solid::DeviceNotifier::deviceRemoved, q, [this](const QString &device) {
-        _k_deviceRemoved(device);
+    QObject::connect(notifier, &Solid::DeviceNotifier::deviceRemoved, q, [this](const QString &device) {
+        deviceRemoved(device);
     });
 
     const QList<Solid::Device> &deviceList = Solid::Device::listFromQuery(predicate);
@@ -732,29 +728,29 @@ void KFilePlacesModel::Private::_k_initDeviceList()
         availableDevices << device.udi();
     }
 
-    _k_reloadBookmarks();
+    reloadBookmarks();
 }
 
-void KFilePlacesModel::Private::_k_deviceAdded(const QString &udi)
+void KFilePlacesModelPrivate::deviceAdded(const QString &udi)
 {
     Solid::Device d(udi);
 
     if (predicate.matches(d)) {
         availableDevices << udi;
-        _k_reloadBookmarks();
+        reloadBookmarks();
     }
 }
 
-void KFilePlacesModel::Private::_k_deviceRemoved(const QString &udi)
+void KFilePlacesModelPrivate::deviceRemoved(const QString &udi)
 {
     auto it = std::find(availableDevices.begin(), availableDevices.end(), udi);
     if (it != availableDevices.end()) {
         availableDevices.erase(it);
-        _k_reloadBookmarks();
+        reloadBookmarks();
     }
 }
 
-void KFilePlacesModel::Private::_k_itemChanged(const QString &id)
+void KFilePlacesModelPrivate::itemChanged(const QString &id)
 {
     for (int row = 0; row < items.size(); ++row) {
         if (items.at(row)->id() == id) {
@@ -764,7 +760,7 @@ void KFilePlacesModel::Private::_k_itemChanged(const QString &id)
     }
 }
 
-void KFilePlacesModel::Private::_k_reloadBookmarks()
+void KFilePlacesModelPrivate::reloadBookmarks()
 {
     QList<KFilePlacesItem *> currentItems = loadBookmarkList();
 
@@ -838,13 +834,13 @@ void KFilePlacesModel::Private::_k_reloadBookmarks()
     Q_EMIT q->reloaded();
 }
 
-bool KFilePlacesModel::Private::isBalooUrl(const QUrl &url) const
+bool KFilePlacesModelPrivate::isBalooUrl(const QUrl &url) const
 {
     const QString scheme = url.scheme();
     return ((scheme == QLatin1String("timeline")) || (scheme == QLatin1String("search")));
 }
 
-QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
+QList<KFilePlacesItem *> KFilePlacesModelPrivate::loadBookmarkList()
 {
     QList<KFilePlacesItem *> items;
 
@@ -875,19 +871,19 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
 
                 KFilePlacesItem *item = nullptr;
                 if (deviceAvailable) {
-                    item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
+                    item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi, q);
                     if (!item->hasSupportedScheme(supportedSchemes)) {
                         delete item;
                         item = nullptr;
                     }
                 } else if (isSupportedScheme && isSupportedUrl && udi.isEmpty() && allowedHere) {
                     // TODO: Update bookmark internal element
-                    item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                    item = new KFilePlacesItem(bookmarkManager, bookmark.address(), QString(), q);
                 }
 
                 if (item) {
-                    connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
-                        _k_itemChanged(id);
+                    QObject::connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
+                        itemChanged(id);
                     });
 
                     items << item;
@@ -896,10 +892,10 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
                 auto it = std::find(tagsList.begin(), tagsList.end(), tag);
                 if (it != tagsList.end()) {
                     tagsList.removeAll(tag);
-                    KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                    KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address(), QString(), q);
                     items << item;
-                    connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
-                        _k_itemChanged(id);
+                    QObject::connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
+                        itemChanged(id);
                     });
                 }
             }
@@ -909,12 +905,12 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
     }
 
     // Add bookmarks for the remaining devices, they were previously unknown
-    for (const QString &udi : qAsConst(devices)) {
+    for (const QString &udi : std::as_const(devices)) {
         bookmark = KFilePlacesItem::createDeviceBookmark(bookmarkManager, udi);
         if (!bookmark.isNull()) {
-            KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
-            connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
-                _k_itemChanged(id);
+            KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi, q);
+            QObject::connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
+                itemChanged(id);
             });
             // TODO: Update bookmark internal element
             items << item;
@@ -924,9 +920,9 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
     for (const QString &tag : tagsList) {
         bookmark = KFilePlacesItem::createTagBookmark(bookmarkManager, tag);
         if (!bookmark.isNull()) {
-            KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address(), tag);
-            connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
-                _k_itemChanged(id);
+            KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address(), tag, q);
+            QObject::connect(item, &KFilePlacesItem::itemChanged, q, [this](const QString &id) {
+                itemChanged(id);
             });
             items << item;
         }
@@ -940,7 +936,7 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
     return items;
 }
 
-int KFilePlacesModel::Private::findNearestPosition(int source, int target)
+int KFilePlacesModelPrivate::findNearestPosition(int source, int target)
 {
     const KFilePlacesItem *item = items.at(source);
     const KFilePlacesModel::GroupType groupType = item->groupType();
@@ -975,7 +971,7 @@ int KFilePlacesModel::Private::findNearestPosition(int source, int target)
     return target;
 }
 
-void KFilePlacesModel::Private::reloadAndSignal()
+void KFilePlacesModelPrivate::reloadAndSignal()
 {
     bookmarkManager->emitChanged(bookmarkManager->root()); // ... we'll get relisted anyway
 }
@@ -1021,7 +1017,7 @@ QMimeData *KFilePlacesModel::mimeData(const QModelIndexList &indexes) const
 
     QDataStream stream(&itemData, QIODevice::WriteOnly);
 
-    for (const QModelIndex &index : qAsConst(indexes)) {
+    for (const QModelIndex &index : std::as_const(indexes)) {
         QUrl itemUrl = url(index);
         if (itemUrl.isValid()) {
             urls << itemUrl;
@@ -1191,20 +1187,25 @@ void KFilePlacesModel::editPlace(const QModelIndex &index, const QString &text,
         return;
     }
 
+    QVector<int> changedRoles;
     bool changed = false;
+
     if (text != bookmark.fullText()) {
         bookmark.setFullText(text);
         changed = true;
+        changedRoles << Qt::DisplayRole;
     }
 
     if (url != bookmark.url()) {
         bookmark.setUrl(url);
         changed = true;
+        changedRoles << KFilePlacesModel::UrlRole;
     }
 
     if (iconName != bookmark.icon()) {
         bookmark.setIcon(iconName);
         changed = true;
+        changedRoles << Qt::DecorationRole;
     }
 
     const QString onlyInApp = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
@@ -1215,7 +1216,7 @@ void KFilePlacesModel::editPlace(const QModelIndex &index, const QString &text,
 
     if (changed) {
         refresh();
-        Q_EMIT dataChanged(index, index);
+        Q_EMIT dataChanged(index, index, changedRoles);
     }
 }
 
@@ -1261,14 +1262,15 @@ void KFilePlacesModel::setPlaceHidden(const QModelIndex &index, bool hidden)
         item->setHidden(hidden);
 
         d->reloadAndSignal();
-        Q_EMIT dataChanged(index, index);
+        Q_EMIT dataChanged(index, index, { KFilePlacesModel::HiddenRole });
     }
 }
 
 void KFilePlacesModel::setGroupHidden(const GroupType type, bool hidden)
 {
-    if (isGroupHidden(type) == hidden)
+    if (isGroupHidden(type) == hidden) {
         return;
+    }
 
     d->bookmarkManager->root().setMetaDataItem(stateNameForGroupType(type), (hidden ? QStringLiteral("true") : QStringLiteral("false")));
     d->reloadAndSignal();
@@ -1317,7 +1319,7 @@ bool KFilePlacesModel::movePlace(int itemRow, int row)
 
     beginMoveRows(QModelIndex(), itemRow, itemRow, QModelIndex(), destRow);
     d->bookmarkManager->root().moveBookmark(bookmark, afterBookmark);
-    // Move item ourselves so that _k_reloadBookmarks() does not consider
+    // Move item ourselves so that reloadBookmarks() does not consider
     // the move as a remove + insert.
     //
     // 2nd argument of QList::move() expects the final destination index,
@@ -1365,15 +1367,14 @@ QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) cons
 
         QString iconName;
         QString text;
-        QString label = data(index, Qt::DisplayRole).toString().replace(QLatin1Char('&'), QLatin1String("&&"));
 
         if (device.is<Solid::OpticalDisc>()) {
-            text = i18n("&Release '%1'", label);
+            text = i18nc("@action:inmenu", "&Release");
         } else if (removable || hotpluggable) {
-            text = i18n("&Safely Remove '%1'", label);
+            text = i18nc("@action:inmenu", "&Safely Remove");
             iconName = QStringLiteral("media-eject");
         } else {
-            text = i18n("&Unmount '%1'", label);
+            text = i18nc("@action:inmenu", "&Unmount");
             iconName = QStringLiteral("media-eject");
         }
 
@@ -1392,8 +1393,7 @@ QAction *KFilePlacesModel::ejectActionForIndex(const QModelIndex &index) const
     Solid::Device device = deviceForIndex(index);
 
     if (device.is<Solid::OpticalDisc>()) {
-        QString label = data(index, Qt::DisplayRole).toString().replace(QLatin1Char('&'), QLatin1String("&&"));
-        QString text = i18n("&Eject '%1'", label);
+        QString text = i18nc("@action:inmenu", "&Eject");
 
         return new QAction(QIcon::fromTheme(QStringLiteral("media-eject")), text, nullptr);
     }
@@ -1408,7 +1408,7 @@ void KFilePlacesModel::requestTeardown(const QModelIndex &index)
 
     if (access != nullptr) {
         connect(access, &Solid::StorageAccess::teardownDone, this, [this](Solid::ErrorType error, QVariant errorData) {
-            d->_k_storageTeardownDone(error, errorData);
+            d->storageTeardownDone(error, errorData);
         });
 
         access->teardown();
@@ -1423,7 +1423,7 @@ void KFilePlacesModel::requestEject(const QModelIndex &index)
 
     if (drive != nullptr) {
         connect(drive, &Solid::OpticalDrive::ejectDone, this, [this](Solid::ErrorType error, QVariant errorData) {
-            d->_k_storageTeardownDone(error, errorData);
+            d->storageTeardownDone(error, errorData);
         });
 
         drive->eject();
@@ -1444,17 +1444,17 @@ void KFilePlacesModel::requestSetup(const QModelIndex &index)
 
         d->setupInProgress[access] = index;
 
-        connect(access, &Solid::StorageAccess::setupDone, this, [this](Solid::ErrorType error, QVariant errorData) {
-            d->_k_storageSetupDone(error, errorData);
+        connect(access, &Solid::StorageAccess::setupDone, this, [this, access](Solid::ErrorType error, QVariant errorData) {
+            d->storageSetupDone(error, errorData, access);
         });
 
         access->setup();
     }
 }
 
-void KFilePlacesModel::Private::_k_storageSetupDone(Solid::ErrorType error, const QVariant &errorData)
+void KFilePlacesModelPrivate::storageSetupDone(Solid::ErrorType error, const QVariant &errorData, Solid::StorageAccess *sender)
 {
-    QPersistentModelIndex index = setupInProgress.take(q->sender());
+    QPersistentModelIndex index = setupInProgress.take(sender);
 
     if (!index.isValid()) {
         return;
@@ -1472,7 +1472,7 @@ void KFilePlacesModel::Private::_k_storageSetupDone(Solid::ErrorType error, cons
     }
 }
 
-void KFilePlacesModel::Private::_k_storageTeardownDone(Solid::ErrorType error, const QVariant &errorData)
+void KFilePlacesModelPrivate::storageTeardownDone(Solid::ErrorType error, const QVariant &errorData)
 {
     if (error && errorData.isValid()) {
         Q_EMIT q->errorMessage(errorData.toString());
@@ -1482,7 +1482,7 @@ void KFilePlacesModel::Private::_k_storageTeardownDone(Solid::ErrorType error, c
 void KFilePlacesModel::setSupportedSchemes(const QStringList &schemes)
 {
     d->supportedSchemes = schemes;
-    d->_k_reloadBookmarks();
+    d->reloadBookmarks();
 }
 
 QStringList KFilePlacesModel::supportedSchemes() const
