@@ -534,11 +534,11 @@ int KFilePlacesViewDelegate::sectionHeaderHeight() const
     return QApplication::fontMetrics().height() + qMax(2, m_view->spacing());
 }
 
-class Q_DECL_HIDDEN KFilePlacesView::Private
+class KFilePlacesViewPrivate
 {
 public:
-    explicit Private(KFilePlacesView *parent)
-        : q(parent)
+    explicit KFilePlacesViewPrivate(KFilePlacesView *qq)
+        : q(qq)
         , m_watcher(new KFilePlacesEventWatcher(q))
         , m_delegate(new KFilePlacesViewDelegate(q))
     {
@@ -571,16 +571,16 @@ public:
     // Adds the "Icon Size" sub-menu items
     void setupIconSizeSubMenu(QMenu *submenu);
 
-    void _k_placeClicked(const QModelIndex &index);
-    void _k_placeEntered(const QModelIndex &index);
-    void _k_placeLeft(const QModelIndex &index);
-    void _k_storageSetupDone(const QModelIndex &index, bool success);
-    void _k_adaptItemsUpdate(qreal value);
-    void _k_itemAppearUpdate(qreal value);
-    void _k_itemDisappearUpdate(qreal value);
-    void _k_enableSmoothItemResizing();
-    void _k_capacityBarFadeValueChanged();
-    void _k_triggerDevicePolling();
+    void placeClicked(const QModelIndex &index);
+    void placeEntered(const QModelIndex &index);
+    void placeLeft(const QModelIndex &index);
+    void storageSetupDone(const QModelIndex &index, bool success);
+    void adaptItemsUpdate(qreal value);
+    void itemAppearUpdate(qreal value);
+    void itemDisappearUpdate(qreal value);
+    void enableSmoothItemResizing();
+    void capacityBarFadeValueChanged(QTimeLine *sender);
+    void triggerDevicePolling();
 
     KFilePlacesView *const q;
 
@@ -616,7 +616,7 @@ public:
 
 KFilePlacesView::KFilePlacesView(QWidget *parent)
     : QListView(parent)
-    , d(new Private(this))
+    , d(std::make_unique<KFilePlacesViewPrivate>(this))
 {
     setItemDelegate(d->m_delegate);
 
@@ -639,28 +639,28 @@ KFilePlacesView::KFilePlacesView(QWidget *parent)
     viewport()->setPalette(palette);
 
     connect(this, &KFilePlacesView::clicked, this, [this](const QModelIndex &index) {
-        d->_k_placeClicked(index);
+        d->placeClicked(index);
     });
     // Note: Don't connect to the activated() signal, as the behavior when it is
     // committed depends on the used widget style. The click behavior of
     // KFilePlacesView should be style independent.
 
     connect(&d->m_adaptItemsTimeline, &QTimeLine::valueChanged, this, [this](qreal value) {
-        d->_k_adaptItemsUpdate(value);
+        d->adaptItemsUpdate(value);
     });
     d->m_adaptItemsTimeline.setDuration(500);
     d->m_adaptItemsTimeline.setUpdateInterval(5);
     d->m_adaptItemsTimeline.setEasingCurve(QEasingCurve::InOutSine);
 
     connect(&d->m_itemAppearTimeline, &QTimeLine::valueChanged, this, [this](qreal value) {
-        d->_k_itemAppearUpdate(value);
+        d->itemAppearUpdate(value);
     });
     d->m_itemAppearTimeline.setDuration(500);
     d->m_itemAppearTimeline.setUpdateInterval(5);
     d->m_itemAppearTimeline.setEasingCurve(QEasingCurve::InOutSine);
 
     connect(&d->m_itemDisappearTimeline, &QTimeLine::valueChanged, this, [this](qreal value) {
-        d->_k_itemDisappearUpdate(value);
+        d->itemDisappearUpdate(value);
     });
     d->m_itemDisappearTimeline.setDuration(500);
     d->m_itemDisappearTimeline.setUpdateInterval(5);
@@ -668,15 +668,15 @@ KFilePlacesView::KFilePlacesView(QWidget *parent)
 
     viewport()->installEventFilter(d->m_watcher);
     connect(d->m_watcher, &KFilePlacesEventWatcher::entryEntered, this, [this](const QModelIndex &index) {
-        d->_k_placeEntered(index);
+        d->placeEntered(index);
     });
     connect(d->m_watcher, &KFilePlacesEventWatcher::entryLeft, this, [this](const QModelIndex &index) {
-        d->_k_placeLeft(index);
+        d->placeLeft(index);
     });
 
     d->m_pollDevices.setInterval(5000);
     connect(&d->m_pollDevices, &QTimer::timeout, this, [this]() {
-        d->_k_triggerDevicePolling();
+        d->triggerDevicePolling();
     });
 
     // FIXME: this is necessary to avoid flashes of black with some widget styles.
@@ -689,7 +689,7 @@ KFilePlacesView::KFilePlacesView(QWidget *parent)
 
 KFilePlacesView::~KFilePlacesView()
 {
-    delete d;
+    viewport()->removeEventFilter(d->m_watcher);
 }
 
 void KFilePlacesView::setDropOnPlaceEnabled(bool enabled)
@@ -789,18 +789,18 @@ void KFilePlacesView::keyPressEvent(QKeyEvent *event)
 {
     QListView::keyPressEvent(event);
     if ((event->key() == Qt::Key_Return) || (event->key() == Qt::Key_Enter)) {
-        d->_k_placeClicked(currentIndex());
+        d->placeClicked(currentIndex());
     }
 }
 
-void KFilePlacesView::Private::readConfig()
+void KFilePlacesViewPrivate::readConfig()
 {
     KConfigGroup cg(KSharedConfig::openConfig(), ConfigGroup);
     m_autoResizeItems = cg.readEntry(PlacesIconsAutoresize, true);
     m_delegate->setIconSize(cg.readEntry(PlacesIconsStaticSize, static_cast<int>(KIconLoader::SizeMedium)));
 }
 
-void KFilePlacesView::Private::writeConfig()
+void KFilePlacesViewPrivate::writeConfig()
 {
     KConfigGroup cg(KSharedConfig::openConfig(), ConfigGroup);
     cg.writeEntry(PlacesIconsAutoresize, m_autoResizeItems);
@@ -891,10 +891,10 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
             properties = menu.addAction(QIcon::fromTheme(QStringLiteral("document-properties")), i18n("Properties"));
         }
         if (!placesModel->isDevice(index)) {
-            edit = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-entry")), i18n("&Edit Entry '%1'...", label));
+            edit = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-entry")), i18nc("@action:inmenu", "&Edit…"));
         }
 
-        hide = menu.addAction(QIcon::fromTheme(QStringLiteral("hint")), i18n("&Hide Entry '%1'", label));
+        hide = menu.addAction(QIcon::fromTheme(QStringLiteral("hint")), i18nc("@action:inmenu", "&Hide"));
         hide->setCheckable(true);
         hide->setChecked(placesModel->isHidden(index));
         // if a parent is hidden no interaction should be possible with children, show it first to do so
@@ -917,7 +917,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 
     QAction *remove = nullptr;
     if (!clickOverHeader && index.isValid() && !placesModel->isDevice(index)) {
-        remove = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-delete")), i18n("&Remove Entry '%1'", label));
+        remove = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-delete")), i18nc("@action:inmenu", "Remove"));
     }
 
     QMenu *iconSizeMenu = new QMenu(i18nc("@item:inmenu", "Icon Size"), &menu);
@@ -961,17 +961,17 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
     } else if (edit && (result == edit)) {
         KBookmark bookmark = placesModel->bookmarkForIndex(index);
         QUrl url = bookmark.url();
-        QString label = bookmark.text();
+        QString bmLabel = bookmark.text();
         QString iconName = bookmark.icon();
         bool appLocal = !bookmark.metaDataItem(QStringLiteral("OnlyInApp")).isEmpty();
 
-        if (KFilePlaceEditDialog::getInformation(true, url, label, iconName, false, appLocal, 64, this)) {
+        if (KFilePlaceEditDialog::getInformation(true, url, bmLabel, iconName, false, appLocal, 64, this)) {
             QString appName;
             if (appLocal) {
                 appName = QCoreApplication::instance()->applicationName();
             }
 
-            placesModel->editPlace(index, label, url, iconName, appName);
+            placesModel->editPlace(index, bmLabel, url, iconName, appName);
         }
 
     } else if (remove && (result == remove)) {
@@ -1000,16 +1000,16 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
         placesModel->requestEject(index);
     } else if (add && (result == add)) {
         QUrl url = d->m_currentUrl;
-        QString label;
+        QString bmLabel;
         QString iconName = QStringLiteral("folder");
         bool appLocal = true;
-        if (KFilePlaceEditDialog::getInformation(true, url, label, iconName, true, appLocal, 64, this)) {
+        if (KFilePlaceEditDialog::getInformation(true, url, bmLabel, iconName, true, appLocal, 64, this)) {
             QString appName;
             if (appLocal) {
                 appName = QCoreApplication::instance()->applicationName();
             }
 
-            placesModel->addPlace(label, url, iconName, appName, index);
+            placesModel->addPlace(bmLabel, url, iconName, appName, index);
         }
     } else if (mount && (result == mount)) {
         placesModel->requestSetup(index);
@@ -1019,7 +1019,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
     selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
 }
 
-void KFilePlacesView::Private::setupIconSizeSubMenu(QMenu *submenu)
+void KFilePlacesViewPrivate::setupIconSizeSubMenu(QMenu *submenu)
 {
     QActionGroup *group = new QActionGroup(submenu);
 
@@ -1090,7 +1090,9 @@ void KFilePlacesView::resizeEvent(QResizeEvent *event)
 void KFilePlacesView::showEvent(QShowEvent *event)
 {
     QListView::showEvent(event);
-    QTimer::singleShot(100, this, SLOT(_k_enableSmoothItemResizing()));
+    QTimer::singleShot(100, this, [this]() {
+        d->enableSmoothItemResizing();
+    });
 }
 
 void KFilePlacesView::hideEvent(QHideEvent *event)
@@ -1285,7 +1287,7 @@ QSize KFilePlacesView::sizeHint() const
     return QSize(iconSize + textWidth + fm.height() / 2, height);
 }
 
-void KFilePlacesView::Private::addDisappearingItem(KFilePlacesViewDelegate *delegate, const QModelIndex &index)
+void KFilePlacesViewPrivate::addDisappearingItem(KFilePlacesViewDelegate *delegate, const QModelIndex &index)
 {
     delegate->addDisappearingItem(index);
     if (m_itemDisappearTimeline.state() != QTimeLine::Running) {
@@ -1294,7 +1296,7 @@ void KFilePlacesView::Private::addDisappearingItem(KFilePlacesViewDelegate *dele
     }
 }
 
-void KFilePlacesView::Private::setCurrentIndex(const QModelIndex &index)
+void KFilePlacesViewPrivate::setCurrentIndex(const QModelIndex &index)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
 
@@ -1316,7 +1318,7 @@ void KFilePlacesView::Private::setCurrentIndex(const QModelIndex &index)
     }
 }
 
-void KFilePlacesView::Private::adaptItemSize()
+void KFilePlacesViewPrivate::adaptItemSize()
 {
     if (!m_autoResizeItems) {
         return;
@@ -1378,7 +1380,7 @@ void KFilePlacesView::Private::adaptItemSize()
     relayoutIconSize(size);
 }
 
-void KFilePlacesView::Private::relayoutIconSize(const int size)
+void KFilePlacesViewPrivate::relayoutIconSize(const int size)
 {
     if (size == m_delegate->iconSize()) {
         return;
@@ -1396,7 +1398,7 @@ void KFilePlacesView::Private::relayoutIconSize(const int size)
     }
 }
 
-void KFilePlacesView::Private::updateHiddenRows()
+void KFilePlacesViewPrivate::updateHiddenRows()
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
 
@@ -1419,7 +1421,7 @@ void KFilePlacesView::Private::updateHiddenRows()
     adaptItemSize();
 }
 
-bool KFilePlacesView::Private::insertAbove(const QRect &itemRect, const QPoint &pos) const
+bool KFilePlacesViewPrivate::insertAbove(const QRect &itemRect, const QPoint &pos) const
 {
     if (m_dropOnPlace) {
         return pos.y() < itemRect.top() + insertIndicatorHeight(itemRect.height()) / 2;
@@ -1428,7 +1430,7 @@ bool KFilePlacesView::Private::insertAbove(const QRect &itemRect, const QPoint &
     return pos.y() < itemRect.top() + (itemRect.height() / 2);
 }
 
-bool KFilePlacesView::Private::insertBelow(const QRect &itemRect, const QPoint &pos) const
+bool KFilePlacesViewPrivate::insertBelow(const QRect &itemRect, const QPoint &pos) const
 {
     if (m_dropOnPlace) {
         return pos.y() > itemRect.bottom() - insertIndicatorHeight(itemRect.height()) / 2;
@@ -1437,7 +1439,7 @@ bool KFilePlacesView::Private::insertBelow(const QRect &itemRect, const QPoint &
     return pos.y() >= itemRect.top() + (itemRect.height() / 2);
 }
 
-int KFilePlacesView::Private::insertIndicatorHeight(int itemHeight) const
+int KFilePlacesViewPrivate::insertIndicatorHeight(int itemHeight) const
 {
     const int min = 4;
     const int max = 12;
@@ -1451,14 +1453,14 @@ int KFilePlacesView::Private::insertIndicatorHeight(int itemHeight) const
     return height;
 }
 
-void KFilePlacesView::Private::fadeCapacityBar(const QModelIndex &index, FadeType fadeType)
+void KFilePlacesViewPrivate::fadeCapacityBar(const QModelIndex &index, FadeType fadeType)
 {
     QTimeLine *timeLine = m_delegate->fadeAnimationForIndex(index);
     delete timeLine;
     m_delegate->removeFadeAnimation(index);
     timeLine = new QTimeLine(250, q);
-    connect(timeLine, &QTimeLine::valueChanged, q, [this]() {
-        _k_capacityBarFadeValueChanged();
+    QObject::connect(timeLine, &QTimeLine::valueChanged, q, [this, timeLine]() {
+        capacityBarFadeValueChanged(timeLine);
     });
     if (fadeType == FadeIn) {
         timeLine->setDirection(QTimeLine::Forward);
@@ -1471,7 +1473,7 @@ void KFilePlacesView::Private::fadeCapacityBar(const QModelIndex &index, FadeTyp
     timeLine->start();
 }
 
-int KFilePlacesView::Private::sectionsCount() const
+int KFilePlacesViewPrivate::sectionsCount() const
 {
     int count = 0;
     QString prevSection;
@@ -1491,7 +1493,7 @@ int KFilePlacesView::Private::sectionsCount() const
     return count;
 }
 
-void KFilePlacesView::Private::triggerItemAppearingAnimation()
+void KFilePlacesViewPrivate::triggerItemAppearingAnimation()
 {
     if (m_itemAppearTimeline.state() != QTimeLine::Running) {
         m_delegate->setAppearingItemProgress(0.0);
@@ -1499,7 +1501,7 @@ void KFilePlacesView::Private::triggerItemAppearingAnimation()
     }
 }
 
-void KFilePlacesView::Private::triggerItemDisappearingAnimation()
+void KFilePlacesViewPrivate::triggerItemDisappearingAnimation()
 {
     if (m_itemDisappearTimeline.state() != QTimeLine::Running) {
         m_delegate->setDisappearingItemProgress(0.0);
@@ -1507,7 +1509,7 @@ void KFilePlacesView::Private::triggerItemDisappearingAnimation()
     }
 }
 
-void KFilePlacesView::Private::_k_placeClicked(const QModelIndex &index)
+void KFilePlacesViewPrivate::placeClicked(const QModelIndex &index)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
 
@@ -1519,7 +1521,7 @@ void KFilePlacesView::Private::_k_placeClicked(const QModelIndex &index)
 
     if (placesModel->setupNeeded(index)) {
         QObject::connect(placesModel, &KFilePlacesModel::setupDone, q, [this](const QModelIndex &idx, bool success) {
-            _k_storageSetupDone(idx, success);
+            storageSetupDone(idx, success);
         });
 
         m_lastClickedIndex = index;
@@ -1530,7 +1532,7 @@ void KFilePlacesView::Private::_k_placeClicked(const QModelIndex &index)
     setCurrentIndex(index);
 }
 
-void KFilePlacesView::Private::_k_placeEntered(const QModelIndex &index)
+void KFilePlacesViewPrivate::placeEntered(const QModelIndex &index)
 {
     fadeCapacityBar(index, FadeIn);
     ++m_pollingRequestCount;
@@ -1539,7 +1541,7 @@ void KFilePlacesView::Private::_k_placeEntered(const QModelIndex &index)
     }
 }
 
-void KFilePlacesView::Private::_k_placeLeft(const QModelIndex &index)
+void KFilePlacesViewPrivate::placeLeft(const QModelIndex &index)
 {
     fadeCapacityBar(index, FadeOut);
     --m_pollingRequestCount;
@@ -1548,7 +1550,7 @@ void KFilePlacesView::Private::_k_placeLeft(const QModelIndex &index)
     }
 }
 
-void KFilePlacesView::Private::_k_storageSetupDone(const QModelIndex &index, bool success)
+void KFilePlacesViewPrivate::storageSetupDone(const QModelIndex &index, bool success)
 {
     if (index != m_lastClickedIndex) {
         return;
@@ -1569,7 +1571,7 @@ void KFilePlacesView::Private::_k_storageSetupDone(const QModelIndex &index, boo
     m_lastClickedIndex = QPersistentModelIndex();
 }
 
-void KFilePlacesView::Private::_k_adaptItemsUpdate(qreal value)
+void KFilePlacesViewPrivate::adaptItemsUpdate(qreal value)
 {
     const int add = (m_endSize - m_oldSize) * value;
     const int size = m_oldSize + add;
@@ -1578,13 +1580,13 @@ void KFilePlacesView::Private::_k_adaptItemsUpdate(qreal value)
     q->scheduleDelayedItemsLayout();
 }
 
-void KFilePlacesView::Private::_k_itemAppearUpdate(qreal value)
+void KFilePlacesViewPrivate::itemAppearUpdate(qreal value)
 {
     m_delegate->setAppearingItemProgress(value);
     q->scheduleDelayedItemsLayout();
 }
 
-void KFilePlacesView::Private::_k_itemDisappearUpdate(qreal value)
+void KFilePlacesViewPrivate::itemDisappearUpdate(qreal value)
 {
     m_delegate->setDisappearingItemProgress(value);
 
@@ -1595,21 +1597,21 @@ void KFilePlacesView::Private::_k_itemDisappearUpdate(qreal value)
     q->scheduleDelayedItemsLayout();
 }
 
-void KFilePlacesView::Private::_k_enableSmoothItemResizing()
+void KFilePlacesViewPrivate::enableSmoothItemResizing()
 {
     m_smoothItemResizing = true;
 }
 
-void KFilePlacesView::Private::_k_capacityBarFadeValueChanged()
+void KFilePlacesViewPrivate::capacityBarFadeValueChanged(QTimeLine *sender)
 {
-    const QModelIndex index = m_delegate->indexForFadeAnimation(static_cast<QTimeLine *>(q->sender()));
+    const QModelIndex index = m_delegate->indexForFadeAnimation(sender);
     if (!index.isValid()) {
         return;
     }
     q->update(index);
 }
 
-void KFilePlacesView::Private::_k_triggerDevicePolling()
+void KFilePlacesViewPrivate::triggerDevicePolling()
 {
     const QModelIndex hoveredIndex = m_watcher->hoveredIndex();
     if (hoveredIndex.isValid()) {
