@@ -56,6 +56,8 @@
 #include <utime.h>
 #endif
 
+#include <set>
+
 static QString expandTilde(const QString &name, bool isfile = false)
 {
     if (name.isEmpty() || name == QLatin1Char('~')) {
@@ -85,7 +87,6 @@ public:
 
     ~KNewFileMenuSingleton()
     {
-        delete dirWatch;
         delete templatesList;
     }
 
@@ -95,20 +96,17 @@ public:
      */
     void parseFiles();
 
-    /**
-     * For entryType
-     * LINKTOTEMPLATE: a desktop file that points to a file or dir to copy
-     * TEMPLATE: a real file to copy as is (the KDE-1.x solution)
-     * SEPARATOR: to put a separator in the menu
-     * 0 means: not parsed, i.e. we don't know
-     */
-    enum EntryType { Unknown, LinkToTemplate = 1, Template, Separator };
+    enum EntryType {
+        Unknown = 0, // Not parsed, i.e. we don't know
+        LinkToTemplate, // A desktop file that points to a file or dir to copy
+        Template, // A real file to copy as is (the KDE-1.x solution)
+    };
 
-    KDirWatch *dirWatch;
+    std::unique_ptr<KDirWatch> dirWatch;
 
     struct Entry {
         QString text;
-        QString filePath; // empty for Separator
+        QString filePath;
         QString templatePath; // same as filePath for Template
         QString icon;
         EntryType entryType;
@@ -142,60 +140,55 @@ void KNewFileMenuSingleton::parseFiles()
 {
     // qDebug();
     filesParsed = true;
-    QMutableListIterator<KNewFileMenuSingleton::Entry> templIter(*templatesList);
+    QMutableListIterator templIter(*templatesList);
     while (templIter.hasNext()) {
         KNewFileMenuSingleton::Entry &templ = templIter.next();
-        const QString filePath = templ.filePath;
-        if (!filePath.isEmpty()) {
-            QString text;
-            QString templatePath;
-            // If a desktop file, then read the name from it.
-            // Otherwise (or if no name in it?) use file name
-            if (KDesktopFile::isDesktopFile(filePath)) {
-                KDesktopFile desktopFile(filePath);
-                if (desktopFile.noDisplay()) {
-                    templIter.remove();
-                    continue;
-                }
-                text = desktopFile.readName();
-                templ.icon = desktopFile.readIcon();
-                templ.comment = desktopFile.readComment();
-                QString type = desktopFile.readType();
-                if (type == QLatin1String("Link")) {
-                    templatePath = desktopFile.desktopGroup().readPathEntry("URL", QString());
-                    if (!templatePath.startsWith(QLatin1Char('/')) && !templatePath.startsWith(QLatin1String("__"))) {
-                        if (templatePath.startsWith(QLatin1String("file:/"))) {
-                            templatePath = QUrl(templatePath).toLocalFile();
-                        } else {
-                            // A relative path, then (that's the default in the files we ship)
-                            const QStringRef linkDir = filePath.leftRef(filePath.lastIndexOf(QLatin1Char('/')) + 1 /*keep / */);
-                            // qDebug() << "linkDir=" << linkDir;
-                            templatePath = linkDir + templatePath;
-                        }
-                    }
-                }
-                if (templatePath.isEmpty()) {
-                    // No URL key, this is an old-style template
-                    templ.entryType = KNewFileMenuSingleton::Template;
-                    templ.templatePath = templ.filePath; // we'll copy the file
-                } else {
-                    templ.entryType = KNewFileMenuSingleton::LinkToTemplate;
-                    templ.templatePath = templatePath;
+        const QString &filePath = templ.filePath;
+        QString text;
+        QString templatePath;
+        // If a desktop file, then read the name from it.
+        // Otherwise (or if no name in it?) use file name
+        if (KDesktopFile::isDesktopFile(filePath)) {
+            KDesktopFile desktopFile(filePath);
+            if (desktopFile.noDisplay()) {
+                templIter.remove();
+                continue;
+            }
+
+            text = desktopFile.readName();
+            templ.icon = desktopFile.readIcon();
+            templ.comment = desktopFile.readComment();
+            if (desktopFile.readType() == QLatin1String("Link")) {
+                templatePath = desktopFile.desktopGroup().readPathEntry("URL", QString());
+                if (templatePath.startsWith(QLatin1String("file:/"))) {
+                    templatePath = QUrl(templatePath).toLocalFile();
+                } else if (!templatePath.startsWith(QLatin1Char('/')) && !templatePath.startsWith(QLatin1String("__"))) {
+                    // A relative path, then (that's the default in the files we ship)
+                    const QStringView linkDir = QStringView(filePath).left(filePath.lastIndexOf(QLatin1Char('/')) + 1 /*keep / */);
+                    // qDebug() << "linkDir=" << linkDir;
+                    templatePath = linkDir + templatePath;
                 }
             }
-            if (text.isEmpty()) {
-                text = QUrl(filePath).fileName();
-                if (text.endsWith(QLatin1String(".desktop"))) {
-                    text.chop(8);
-                }
+            if (templatePath.isEmpty()) {
+                // No URL key, this is an old-style template
+                templ.entryType = KNewFileMenuSingleton::Template;
+                templ.templatePath = templ.filePath; // we'll copy the file
+            } else {
+                templ.entryType = KNewFileMenuSingleton::LinkToTemplate;
+                templ.templatePath = templatePath;
             }
-            templ.text = text;
-            /*// qDebug() << "Updating entry with text=" << text
-                          << "entryType=" << templ.entryType
-                          << "templatePath=" << templ.templatePath;*/
-        } else {
-            templ.entryType = KNewFileMenuSingleton::Separator;
         }
+        if (text.isEmpty()) {
+            text = QUrl(filePath).fileName();
+            const QLatin1String suffix(".desktop");
+            if (text.endsWith(suffix)) {
+                text.chop(suffix.size());
+            }
+        }
+        templ.text = text;
+        /*// qDebug() << "Updating entry with text=" << text
+                        << "entryType=" << templ.entryType
+                        << "templatePath=" << templ.templatePath;*/
     }
 }
 
@@ -282,12 +275,12 @@ public:
     /**
      * Just clears the string buffer d->m_text, but I need a slot for this to occur
      */
-    void _k_slotAbortDialog();
+    void slotAbortDialog();
 
     /**
      * Called when New->* is clicked
      */
-    void _k_slotActionTriggered(QAction *action);
+    void slotActionTriggered(QAction *action);
 
     /**
      * Shows a dialog asking the user to enter a name when creating a new folder.
@@ -297,34 +290,28 @@ public:
     /**
      * Callback function that reads in directory name from dialog and processes it
      */
-    void _k_slotCreateDirectory();
-
-    /**
-     * Callback function that reads in directory name from dialog and processes it. This will wirte
-     * a hidden directory without further questions
-     */
-    void _k_slotCreateHiddenDirectory();
+    void slotCreateDirectory();
 
     /**
      * Fills the templates list.
      */
-    void _k_slotFillTemplates();
+    void slotFillTemplates();
 
     /**
      * Called when accepting the KPropertiesDialog (for "other desktop files")
      */
-    void _k_slotOtherDesktopFile();
+    void _k_slotOtherDesktopFile(KPropertiesDialog *sender);
 
     /**
      * Called when closing the KPropertiesDialog is closed (whichever way, accepted and rejected)
      */
-    void _k_slotOtherDesktopFileClosed();
+    void slotOtherDesktopFileClosed();
 
     /**
      * Callback in KNewFileMenu for the RealFile Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
-    void _k_slotRealFileOrDir();
+    void slotRealFileOrDir();
 
     /**
      * Delay calls to _k_slotTextChanged
@@ -341,13 +328,13 @@ public:
      * Callback in KNewFileMenu for the Symlink Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
-    void _k_slotSymLink();
+    void slotSymLink();
 
     /**
      * Callback in KNewFileMenu for the Url/Desktop Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
-    void _k_slotUrlDesktopFile();
+    void slotUrlDesktopFile();
 
     /**
      * Callback to check if a file/directory with the same name as the one being created, exists
@@ -475,11 +462,11 @@ bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
         dialog->setModal(q->isModal());
         dialog->setAttribute(Qt::WA_DeleteOnClose);
 
-        m_buttonBox = new QDialogButtonBox(dialog);
-        m_buttonBox->setStandardButtons(QDialogButtonBox::Ok);
+        QDialogButtonBox *box = new QDialogButtonBox(dialog);
+        box->setStandardButtons(QDialogButtonBox::Ok);
 
         KMessageBox::createKMessageBox(dialog,
-                                       m_buttonBox,
+                                       box,
                                        QMessageBox::Warning,
                                        i18n("<qt>The template file <b>%1</b> does not exist.</qt>", src),
                                        QStringList(),
@@ -500,8 +487,7 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         return;
     }
 
-    QList<QUrl>::const_iterator it = m_popupFiles.constBegin();
-    for (; it != m_popupFiles.constEnd(); ++it) {
+    for (const auto &url : std::as_const(m_popupFiles)) {
         QString text = entry.text;
         text.remove(QStringLiteral("...")); // the ... is fine for the menu item but not for the default filename
         text = text.trimmed(); // In some languages, there is a space in front of "...", see bug 268895
@@ -510,7 +496,7 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         QString name = text;
         text.append(QStringLiteral(".desktop"));
 
-        const QUrl directory = mostLocalUrl(*it);
+        const QUrl directory = mostLocalUrl(url);
         const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + QLatin1Char('/') + KIO::encodeFileName(text));
         if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
             text = KFileUtils::suggestName(directory, text);
@@ -536,12 +522,12 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         KPropertiesDialog *dlg = new KPropertiesDialog(templateUrl, directory, text, m_parentWidget);
         dlg->setModal(q->isModal());
         dlg->setAttribute(Qt::WA_DeleteOnClose);
-        QObject::connect(dlg, &KPropertiesDialog::applied, q, [this]() {
-            _k_slotOtherDesktopFile();
+        QObject::connect(dlg, &KPropertiesDialog::applied, q, [this, dlg]() {
+            _k_slotOtherDesktopFile(dlg);
         });
         if (usingTemplate) {
             QObject::connect(dlg, &KPropertiesDialog::propertiesClosed, q, [this]() {
-                _k_slotOtherDesktopFileClosed();
+                slotOtherDesktopFileClosed();
             });
         }
         dlg->show();
@@ -559,9 +545,9 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
     text.remove(QStringLiteral("...")); // the ... is fine for the menu item but not for the default filename
     text = text.trimmed(); // In some languages, there is a space in front of "...", see bug 268895
     // add the extension (from the templatePath), should work with .txt, .html and with ".tar.gz"... etc
-    const QString fileName = entry.templatePath.mid(entry.templatePath.lastIndexOf(QLatin1Char('/')));
+    const QStringView fileName = QStringView(entry.templatePath).mid(entry.templatePath.lastIndexOf(QLatin1Char('/')));
     const int dotIndex = fileName.indexOf(QLatin1Char('.'));
-    text += dotIndex > 0 ? fileName.midRef(dotIndex) : QStringRef();
+    text += dotIndex > 0 ? fileName.mid(dotIndex) : QStringView{};
 
     m_copyData.m_src = entry.templatePath;
 
@@ -586,10 +572,10 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
     });
 
     QObject::connect(m_fileDialog, &QDialog::accepted, q, [this]() {
-        _k_slotRealFileOrDir();
+        slotRealFileOrDir();
     });
     QObject::connect(m_fileDialog, &QDialog::rejected, q, [this]() {
-        _k_slotAbortDialog();
+        slotAbortDialog();
     });
 
     m_fileDialog->show();
@@ -599,13 +585,13 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
 
 void KNewFileMenuPrivate::executeSymLink(const KNewFileMenuSingleton::Entry &entry)
 {
-    KNameAndUrlInputDialog *dlg = new KNameAndUrlInputDialog(i18n("File name:"), entry.comment, m_popupFiles.first(), m_parentWidget);
+    KNameAndUrlInputDialog *dlg = new KNameAndUrlInputDialog(i18n("Name for new link:"), entry.comment, m_popupFiles.first(), m_parentWidget);
     dlg->setModal(q->isModal());
     dlg->setAttribute(Qt::WA_DeleteOnClose);
     dlg->setWindowTitle(i18n("Create Symlink"));
     m_fileDialog = dlg;
     QObject::connect(dlg, &QDialog::accepted, q, [this]() {
-        _k_slotSymLink();
+        slotSymLink();
     });
     dlg->show();
 }
@@ -637,18 +623,17 @@ void KNewFileMenuPrivate::executeStrategy()
             QMimeType wantedMime = db.mimeTypeForUrl(uSrc);
             QMimeType mime = db.mimeTypeForFileNameAndData(m_copyData.m_chosenFileName, srcFile.read(1024));
             // qDebug() << "mime=" << mime->name() << "wantedMime=" << wantedMime->name();
-            if (!mime.inherits(wantedMime.name()))
+            if (!mime.inherits(wantedMime.name())) {
                 if (!wantedMime.preferredSuffix().isEmpty()) {
                     chosenFileName += QLatin1Char('.') + wantedMime.preferredSuffix();
                 }
+            }
         }
     }
 
-    // The template is not a desktop file [or it's a URL one]
-    // Copy it.
-    QList<QUrl>::const_iterator it = m_popupFiles.constBegin();
-    for (; it != m_popupFiles.constEnd(); ++it) {
-        QUrl dest = *it;
+    // The template is not a desktop file [or it's a URL one] >>> Copy it
+    for (const auto &u : std::as_const(m_popupFiles)) {
+        QUrl dest = u;
         dest.setPath(concatPaths(dest.path(), KIO::encodeFileName(chosenFileName)));
 
         QList<QUrl> lstSrc;
@@ -682,14 +667,14 @@ void KNewFileMenuPrivate::executeStrategy()
 
 void KNewFileMenuPrivate::executeUrlDesktopFile(const KNewFileMenuSingleton::Entry &entry)
 {
-    KNameAndUrlInputDialog *dlg = new KNameAndUrlInputDialog(i18n("File name:"), entry.comment, m_popupFiles.first(), m_parentWidget);
+    KNameAndUrlInputDialog *dlg = new KNameAndUrlInputDialog(i18n("Name for new link:"), entry.comment, m_popupFiles.first(), m_parentWidget);
     m_copyData.m_templatePath = entry.templatePath;
     dlg->setModal(q->isModal());
     dlg->setAttribute(Qt::WA_DeleteOnClose);
     dlg->setWindowTitle(i18n("Create link to URL"));
     m_fileDialog = dlg;
     QObject::connect(dlg, &QDialog::accepted, q, [this]() {
-        _k_slotUrlDesktopFile();
+        slotUrlDesktopFile();
     });
     dlg->show();
 }
@@ -701,7 +686,7 @@ void KNewFileMenuPrivate::fillMenu()
     m_menuDev->menu()->clear();
     m_newDirAction = nullptr;
 
-    QSet<QString> seenTexts;
+    std::set<QString> seenTexts;
     QString lastTemplatePath;
     // these shall be put at special positions
     QAction *linkURL = nullptr;
@@ -709,23 +694,17 @@ void KNewFileMenuPrivate::fillMenu()
     QAction *linkPath = nullptr;
 
     KNewFileMenuSingleton *s = kNewMenuGlobals();
-    int i = 1;
-    KNewFileMenuSingleton::EntryList::iterator templ = s->templatesList->begin();
-    const KNewFileMenuSingleton::EntryList::iterator templ_end = s->templatesList->end();
-    for (; templ != templ_end; ++templ, ++i) {
-        KNewFileMenuSingleton::Entry &entry = *templ;
-        if (entry.entryType != KNewFileMenuSingleton::Separator) {
+    int idx = 0;
+    for (auto &entry : *s->templatesList) {
+        ++idx;
+        if (entry.entryType != KNewFileMenuSingleton::Unknown) {
             // There might be a .desktop for that one already, if it's a kdelnk
             // This assumes we read .desktop files before .kdelnk files ...
 
             // In fact, we skip any second item that has the same text as another one.
             // Duplicates in a menu look bad in any case.
-
-            const bool bSkip = seenTexts.contains(entry.text);
-            if (bSkip) {
-                // qDebug() << "skipping" << entry.filePath;
-            } else {
-                seenTexts.insert(entry.text);
+            const auto [it, isInserted] = seenTexts.insert(entry.text);
+            if (isInserted) {
                 // const KNewFileMenuSingleton::Entry entry = templatesList->at(i-1);
 
                 const QString templatePath = entry.templatePath;
@@ -773,7 +752,7 @@ void KNewFileMenuPrivate::fillMenu()
                             } else {
                                 mime = db.mimeTypeForName(entry.mimeType);
                             }
-                            for (const QString &supportedMime : qAsConst(m_supportedMimeTypes)) {
+                            for (const QString &supportedMime : std::as_const(m_supportedMimeTypes)) {
                                 if (mime.inherits(supportedMime)) {
                                     keep = true;
                                     break;
@@ -788,7 +767,7 @@ void KNewFileMenuPrivate::fillMenu()
                     }
 
                     QAction *act = new QAction(q);
-                    act->setData(i);
+                    act->setData(idx);
                     act->setIcon(QIcon::fromTheme(entry.icon));
                     act->setText(i18nc("@item:inmenu Create New", "%1", entry.text));
                     act->setActionGroup(m_newMenuGroup);
@@ -864,12 +843,12 @@ QUrl KNewFileMenuPrivate::mostLocalUrl(const QUrl &url)
     return job->exec() ? job->mostLocalUrl() : url;
 }
 
-void KNewFileMenuPrivate::_k_slotAbortDialog()
+void KNewFileMenuPrivate::slotAbortDialog()
 {
     m_text = QString();
 }
 
-void KNewFileMenuPrivate::_k_slotActionTriggered(QAction *action)
+void KNewFileMenuPrivate::slotActionTriggered(QAction *action)
 {
     q->trigger(); // was for kdesktop's slotNewMenuActivated() in kde3 times. Can't hurt to keep it...
 
@@ -902,7 +881,7 @@ void KNewFileMenuPrivate::_k_slotActionTriggered(QAction *action)
     }
 }
 
-void KNewFileMenuPrivate::_k_slotCreateDirectory()
+void KNewFileMenuPrivate::slotCreateDirectory()
 {
     // Automatically trim trailing spaces since they're pretty much always
     // unintentional and can cause issues on Windows in shared environments
@@ -919,40 +898,6 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory()
         if (isAbsoluteLocalPath(name)) {
             url = QUrl::fromLocalFile(name);
         } else {
-            if (name == QLatin1Char('.') || name == QLatin1String("..")) {
-                KGuiItem enterNewNameGuiItem(KStandardGuiItem::ok());
-                enterNewNameGuiItem.setText(i18nc("@action:button", "Enter a Different Name"));
-                enterNewNameGuiItem.setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
-
-                QDialog *confirmDialog = new QDialog(m_parentWidget);
-                confirmDialog->setWindowTitle(i18n("Invalid Directory Name"));
-                confirmDialog->setModal(m_modal);
-                confirmDialog->setAttribute(Qt::WA_DeleteOnClose);
-
-                m_buttonBox = new QDialogButtonBox(confirmDialog);
-                m_buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-                KGuiItem::assign(m_buttonBox->button(QDialogButtonBox::Ok), enterNewNameGuiItem);
-
-                KMessageBox::createKMessageBox(
-                    confirmDialog,
-                    m_buttonBox,
-                    QMessageBox::Critical,
-                    xi18nc("@info",
-                           "Could not create a folder with the name <filename>%1</filename><nl/>because it is reserved for use by the operating system.",
-                           name),
-                    QStringList(),
-                    QString(),
-                    nullptr,
-                    KMessageBox::NoExec,
-                    QString());
-
-                m_creatingDirectory = true;
-                QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, q, &KNewFileMenu::createDirectory);
-                m_fileDialog = confirmDialog;
-                confirmDialog->show();
-                _k_slotAbortDialog();
-                return;
-            }
             url = baseUrl;
             url.setPath(concatPaths(url.path(), name));
         }
@@ -977,79 +922,51 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory()
         job->uiDelegate()->setAutoErrorHandlingEnabled(false);
         QObject::connect(job, &KJob::result, q, &KNewFileMenu::slotResult);
     }
-    _k_slotAbortDialog();
+    slotAbortDialog();
 }
 
-void KNewFileMenuPrivate::_k_slotCreateHiddenDirectory()
-{
-    _k_slotCreateDirectory();
-}
-
-struct EntryWithName {
+struct EntryInfo {
     QString key;
+    QString url;
     KNewFileMenuSingleton::Entry entry;
 };
 
-void KNewFileMenuPrivate::_k_slotFillTemplates()
+static QStringList getInstalledTemplates()
 {
-    KNewFileMenuSingleton *s = kNewMenuGlobals();
-    // qDebug();
-
-    const QStringList qrcTemplates = {QStringLiteral(":/kio5/newfile-templates")};
-    QStringList installedTemplates = {
-        QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("templates"), QStandardPaths::LocateDirectory)};
-// Qt does not provide an easy way to receive the xdg dir for templates so we have to find it on our own
+    QStringList list = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("templates"), QStandardPaths::LocateDirectory);
+    // TODO KF6, use QStandardPaths::TemplatesLocations
 #ifdef Q_OS_UNIX
     QString xdgUserDirs = QStandardPaths::locate(QStandardPaths::ConfigLocation, QStringLiteral("user-dirs.dirs"), QStandardPaths::LocateFile);
     QFile xdgUserDirsFile(xdgUserDirs);
     if (!xdgUserDirs.isEmpty() && xdgUserDirsFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        static const QLatin1String marker("XDG_TEMPLATES_DIR=");
+        QString line;
         QTextStream in(&xdgUserDirsFile);
         while (!in.atEnd()) {
-            QString line = in.readLine();
-            if (line.startsWith(QLatin1String("XDG_TEMPLATES_DIR="))) {
-                QString xdgTemplates = line.mid(19, line.size() - 20);
-                xdgTemplates.replace(QStringLiteral("$HOME"), QDir::homePath());
-                QDir xdgTemplatesDir(xdgTemplates);
-                if (xdgTemplatesDir.exists()) {
-                    installedTemplates << xdgTemplates;
+            line.clear();
+            in.readLineInto(&line);
+            if (line.startsWith(marker)) {
+                // E.g. XDG_TEMPLATES_DIR="$HOME/templates" -> $HOME/templates
+                line.remove(0, marker.size()).remove(QLatin1Char('"'));
+                line.replace(QLatin1String("$HOME"), QDir::homePath());
+                if (QDir(line).exists()) {
+                    list << line;
                 }
                 break;
             }
         }
     }
 #endif
+    return list;
+}
 
-    const QStringList templates = qrcTemplates + installedTemplates;
-
-    // Ensure any changes in the templates dir will call this
-    if (!s->dirWatch) {
-        s->dirWatch = new KDirWatch;
-        for (const QString &dir : qAsConst(installedTemplates)) {
-            s->dirWatch->addDir(dir);
-        }
-        QObject::connect(s->dirWatch, &KDirWatch::dirty, q, [this]() {
-            _k_slotFillTemplates();
-        });
-        QObject::connect(s->dirWatch, &KDirWatch::created, q, [this]() {
-            _k_slotFillTemplates();
-        });
-        QObject::connect(s->dirWatch, &KDirWatch::deleted, q, [this]() {
-            _k_slotFillTemplates();
-        });
-        // Ok, this doesn't cope with new dirs in XDG_DATA_DIRS, but that's another story
-    }
-    ++s->templatesVersion;
-    s->filesParsed = false;
-
-    s->templatesList->clear();
-
-    // Look into "templates" dirs.
-    QStringList files;
+static QStringList getTemplateFilePaths(const QStringList &templates)
+{
     QDir dir;
-
+    QStringList files;
     for (const QString &path : templates) {
         dir.setPath(path);
-        const QStringList &entryList(dir.entryList(QStringList{QStringLiteral("*.desktop")}, QDir::Files));
+        const QStringList entryList = dir.entryList(QStringList{QStringLiteral("*.desktop")}, QDir::Files);
         files.reserve(files.size() + entryList.size());
         for (const QString &entry : entryList) {
             const QString file = concatPaths(dir.path(), entry);
@@ -1057,59 +974,105 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
         }
     }
 
-    QMap<QString, KNewFileMenuSingleton::Entry> slist; // used for sorting
-    QMap<QString, EntryWithName> ulist; // entries with unique URLs
-    for (const QString &file : qAsConst(files)) {
-        // qDebug() << file;
-        if (file[0] != QLatin1Char('.')) {
-            KNewFileMenuSingleton::Entry e;
-            e.filePath = file;
-            e.entryType = KNewFileMenuSingleton::Unknown; // not parsed yet
+    return files;
+}
 
-            // Put Directory first in the list (a bit hacky),
-            // and TextFile before others because it's the most used one.
-            // This also sorts by user-visible name.
-            // The rest of the re-ordering is done in fillMenu.
-            const KDesktopFile config(file);
-            QString url = config.desktopGroup().readEntry("URL");
-            QString key = config.desktopGroup().readEntry("Name");
-            if (file.endsWith(QLatin1String("Directory.desktop"))) {
-                key.prepend(QLatin1Char('0'));
-            } else if (file.startsWith(QDir::homePath())) {
-                key.prepend(QLatin1Char('1'));
-            } else if (file.endsWith(QLatin1String("TextFile.desktop"))) {
-                key.prepend(QLatin1Char('2'));
-            } else {
-                key.prepend(QLatin1Char('3'));
-            }
-            EntryWithName en = {key, e};
-            if (ulist.contains(url)) {
-                ulist.remove(url);
-            }
-            ulist.insert(url, en);
+void KNewFileMenuPrivate::slotFillTemplates()
+{
+    KNewFileMenuSingleton *instance = kNewMenuGlobals();
+    // qDebug();
+
+    const QStringList installedTemplates = getInstalledTemplates();
+    const QStringList qrcTemplates{QStringLiteral(":/kio5/newfile-templates")};
+    const QStringList templates = qrcTemplates + installedTemplates;
+
+    // Ensure any changes in the templates dir will call this
+    if (!instance->dirWatch) {
+        instance->dirWatch = std::make_unique<KDirWatch>();
+        for (const QString &dir : installedTemplates) {
+            instance->dirWatch->addDir(dir);
+        }
+
+        auto slotFunc = [this]() {
+            slotFillTemplates();
+        };
+        QObject::connect(instance->dirWatch.get(), &KDirWatch::dirty, q, slotFunc);
+        QObject::connect(instance->dirWatch.get(), &KDirWatch::created, q, slotFunc);
+        QObject::connect(instance->dirWatch.get(), &KDirWatch::deleted, q, slotFunc);
+        // Ok, this doesn't cope with new dirs in XDG_DATA_DIRS, but that's another story
+    }
+
+    // Look into "templates" dirs.
+    QStringList files = getTemplateFilePaths(templates);
+    auto removeFunc = [](const QString &path) {
+        return path.startsWith(QLatin1Char('.'));
+    };
+    files.erase(std::remove_if(files.begin(), files.end(), removeFunc), files.end());
+
+    std::vector<EntryInfo> uniqueEntries;
+
+    for (const QString &file : files) {
+        // qDebug() << file;
+        KNewFileMenuSingleton::Entry entry;
+        entry.filePath = file;
+        entry.entryType = KNewFileMenuSingleton::Unknown; // not parsed yet
+
+        // Put Directory first in the list (a bit hacky),
+        // and TextFile before others because it's the most used one.
+        // This also sorts by user-visible name.
+        // The rest of the re-ordering is done in fillMenu.
+        const KDesktopFile config(file);
+        const QString url = config.desktopGroup().readEntry("URL");
+        QString key = config.desktopGroup().readEntry("Name");
+        if (file.endsWith(QLatin1String("Directory.desktop"))) {
+            key.prepend(QLatin1Char('0'));
+        } else if (file.startsWith(QDir::homePath())) {
+            key.prepend(QLatin1Char('1'));
+        } else if (file.endsWith(QLatin1String("TextFile.desktop"))) {
+            key.prepend(QLatin1Char('2'));
+        } else {
+            key.prepend(QLatin1Char('3'));
+        }
+
+        EntryInfo eInfo = {key, url, entry};
+        auto it = std::find_if(uniqueEntries.begin(), uniqueEntries.end(), [&url](const EntryInfo &info) {
+            return url == info.url;
+        });
+
+        if (it != uniqueEntries.cend()) {
+            *it = eInfo;
+        } else {
+            uniqueEntries.push_back(eInfo);
         }
     }
-    QMap<QString, EntryWithName>::iterator it = ulist.begin();
-    for (; it != ulist.end(); ++it) {
-        EntryWithName ewn = *it;
-        slist.insert(ewn.key, ewn.entry);
-    }
-    (*s->templatesList) += slist.values();
+
+    std::sort(uniqueEntries.begin(), uniqueEntries.end(), [](const EntryInfo &a, const EntryInfo &b) {
+        return a.key < b.key;
+    });
+
+    ++instance->templatesVersion;
+    instance->filesParsed = false;
+
+    instance->templatesList->clear();
+
+    instance->templatesList->reserve(uniqueEntries.size());
+    for (const auto &info : uniqueEntries) {
+        instance->templatesList->append(info.entry);
+    };
 }
 
-void KNewFileMenuPrivate::_k_slotOtherDesktopFile()
+void KNewFileMenuPrivate::_k_slotOtherDesktopFile(KPropertiesDialog *sender)
 {
     // The properties dialog took care of the copying, so we're done
-    KPropertiesDialog *dialog = qobject_cast<KPropertiesDialog *>(q->sender());
-    Q_EMIT q->fileCreated(dialog->url());
+    Q_EMIT q->fileCreated(sender->url());
 }
 
-void KNewFileMenuPrivate::_k_slotOtherDesktopFileClosed()
+void KNewFileMenuPrivate::slotOtherDesktopFileClosed()
 {
     QFile::remove(m_tempFileToDelete);
 }
 
-void KNewFileMenuPrivate::_k_slotRealFileOrDir()
+void KNewFileMenuPrivate::slotRealFileOrDir()
 {
     // Automatically trim trailing spaces since they're pretty much always
     // unintentional and can cause issues on Windows in shared environments
@@ -1117,11 +1080,11 @@ void KNewFileMenuPrivate::_k_slotRealFileOrDir()
         m_text.chop(1);
     }
     m_copyData.m_chosenFileName = m_text;
-    _k_slotAbortDialog();
+    slotAbortDialog();
     executeStrategy();
 }
 
-void KNewFileMenuPrivate::_k_slotSymLink()
+void KNewFileMenuPrivate::slotSymLink()
 {
     KNameAndUrlInputDialog *dlg = static_cast<KNameAndUrlInputDialog *>(m_fileDialog);
 
@@ -1190,7 +1153,7 @@ void KNewFileMenuPrivate::_k_slotTextChanged(const QString &text)
             if (folders.count() > 1) {
                 label = i18n("Using slashes in folder names will create sub-folders, like so:");
                 QString indentation = QString();
-                for (const QString &folder : qAsConst(folders)) {
+                for (const QString &folder : std::as_const(folders)) {
                     label.append(QLatin1Char('\n'));
                     label.append(indentation);
                     label.append(folder);
@@ -1242,7 +1205,7 @@ void KNewFileMenuPrivate::_k_slotTextChanged(const QString &text)
         } else {
             url = QUrl(m_baseUrl.toString() + QLatin1Char('/') + text);
         }
-        KIO::StatJob *job = KIO::statDetails(url, KIO::StatJob::StatSide::DestinationSide, KIO::StatDetail::StatBasic);
+        KIO::StatJob *job = KIO::statDetails(url, KIO::StatJob::StatSide::DestinationSide, KIO::StatDetail::StatBasic, KIO::HideProgressInfo);
         QObject::connect(job, &KJob::result, q, [this](KJob *job) {
             _k_slotStatResult(job);
         });
@@ -1307,7 +1270,7 @@ void KNewFileMenuPrivate::_k_slotStatResult(KJob *job)
     }
 }
 
-void KNewFileMenuPrivate::_k_slotUrlDesktopFile()
+void KNewFileMenuPrivate::slotUrlDesktopFile()
 {
     KNameAndUrlInputDialog *dlg = static_cast<KNameAndUrlInputDialog *>(m_fileDialog);
 
@@ -1377,13 +1340,13 @@ void KNewFileMenuPrivate::_k_slotUrlDesktopFile()
 
 KNewFileMenu::KNewFileMenu(KActionCollection *collection, const QString &name, QObject *parent)
     : KActionMenu(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Create New"), parent)
-    , d(new KNewFileMenuPrivate(collection, this))
+    , d(std::make_unique<KNewFileMenuPrivate>(collection, this))
 {
     // Don't fill the menu yet
     // We'll do that in checkUpToDate (should be connected to aboutToShow)
     d->m_newMenuGroup = new QActionGroup(this);
     connect(d->m_newMenuGroup, &QActionGroup::triggered, this, [this](QAction *action) {
-        d->_k_slotActionTriggered(action);
+        d->slotActionTriggered(action);
     });
     d->m_parentWidget = qobject_cast<QWidget *>(parent);
     d->m_newDirAction = nullptr;
@@ -1395,11 +1358,7 @@ KNewFileMenu::KNewFileMenu(KActionCollection *collection, const QString &name, Q
     d->m_menuDev = new KActionMenu(QIcon::fromTheme(QStringLiteral("drive-removable-media")), i18n("Link to Device"), this);
 }
 
-KNewFileMenu::~KNewFileMenu()
-{
-    // qDebug() << this;
-    delete d;
-}
+KNewFileMenu::~KNewFileMenu() = default;
 
 void KNewFileMenu::checkUpToDate()
 {
@@ -1414,7 +1373,7 @@ void KNewFileMenu::checkUpToDate()
 
         if (!s->templatesList) { // No templates list up to now
             s->templatesList = new KNewFileMenuSingleton::EntryList;
-            d->_k_slotFillTemplates();
+            d->slotFillTemplates();
             s->parseFiles();
         }
 
@@ -1471,10 +1430,10 @@ void KNewFileMenuPrivate::showNewDirNameDlg(const QString &name)
     });
 
     QObject::connect(m_fileDialog, &QDialog::accepted, q, [this]() {
-        _k_slotCreateDirectory();
+        slotCreateDirectory();
     });
     QObject::connect(m_fileDialog, &QDialog::rejected, q, [this]() {
-        _k_slotAbortDialog();
+        slotAbortDialog();
     });
 
     m_fileDialog->show();
