@@ -38,6 +38,7 @@
 #endif
 #include <algorithm>
 #include <kio_widgets_debug.h>
+#include <set>
 
 static bool KIOSKAuthorizedAction(const KConfigGroup &cfg)
 {
@@ -208,10 +209,7 @@ KFileItemActions::KFileItemActions(QObject *parent)
 {
 }
 
-KFileItemActions::~KFileItemActions()
-{
-    delete d;
-}
+KFileItemActions::~KFileItemActions() = default;
 
 void KFileItemActions::setItemListProperties(const KFileItemListProperties &itemListProperties)
 {
@@ -606,11 +604,9 @@ QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu
 
             ServiceList &list = s.selectList(priority, submenuName);
             const ServiceList userServices = KDesktopFileActions::userDefinedServices(KService(file), isLocal, urlList);
-            for (const KServiceAction &action : userServices) {
-                if (showGroup.readEntry(action.name(), true) && !excludeList.contains(action.name())) {
-                    list += action;
-                }
-            }
+            std::copy_if(userServices.cbegin(), userServices.cend(), std::back_inserter(list), [&excludeList, &showGroup](const KServiceAction &srvAction) {
+                return showGroup.readEntry(srvAction.name(), true) && !excludeList.contains(srvAction.name());
+            });
         }
     }
 
@@ -696,7 +692,6 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu, QMenu *actionsM
     QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
     const KService::List fileItemPlugins =
         KMimeTypeTrader::self()->query(commonMimeType, QStringLiteral("KFileItemAction/Plugin"), QStringLiteral("exist Library"));
-    QT_WARNING_POP
     for (const auto &service : fileItemPlugins) {
         if (!showGroup.readEntry(service->desktopEntryName(), true)) {
             // The plugin has been disabled
@@ -726,6 +721,7 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu, QMenu *actionsM
             addedPlugins.append(service->desktopEntryName());
         }
     }
+    QT_WARNING_POP
 #endif
 
     return itemCount;
@@ -861,7 +857,7 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
         QAction *runAct = new QAction(this);
         if (serviceIdList.count() == 1) {
             const KService::Ptr app = preferredService(m_mimeTypeList.first(), excludedDesktopEntryNames, traderConstraint);
-            runAct->setText(i18n("&Open with %1", app->name()));
+            runAct->setText(isDir ? i18n("&Open folder with %1", app->name()) : i18n("&Open with %1", app->name()));
             runAct->setIcon(QIcon::fromTheme(app->icon()));
 
             // Remove that app from the offers list (#242731)
@@ -883,7 +879,10 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
     }
 
     QAction *openWithAct = new QAction(this);
-    openWithAct->setText(i18nc("@title:menu", "&Open With..."));
+    openWithAct->setText(
+        isDir ?
+            i18nc("@title:menu", "&Open Folder With...") :
+            i18nc("@title:menu", "&Open With..."));
     openWithAct->setObjectName(QStringLiteral("openwith_browse")); // For the unittest
     QObject::connect(openWithAct, &QAction::triggered, this, &KFileItemActionsPrivate::slotOpenWithDialog);
 
@@ -896,7 +895,7 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
 
         // If there are still more apps, show them in a sub-menu
         if (!offers.isEmpty()) { // submenu 'open with'
-            QMenu *subMenu = new QMenu(i18nc("@title:menu", "&Open With"), topMenu);
+            QMenu *subMenu = new QMenu(isDir ? i18nc("@title:menu", "&Open Folder With") : i18nc("@title:menu", "&Open With"), topMenu);
             subMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
             subMenu->menuAction()->setObjectName(QStringLiteral("openWith_submenu")); // For the unittest
             // Add other apps to the sub-menu
@@ -927,16 +926,23 @@ QStringList KFileItemActionsPrivate::serviceMenuFilePaths()
     QStringList filePaths;
 
     // Use old KServiceTypeTrader code path
+    std::set<QString> uniqueFileNames;
 #if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 85)
     const KService::List entries = KServiceTypeTrader::self()->query(QStringLiteral("KonqPopupMenu/Plugin"));
     for (const KServicePtr &entry : entries) {
         filePaths << QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + entry->entryPath());
+        uniqueFileNames.insert(entry->entryPath().split(QLatin1Char('/')).last());
     }
 #endif
     // Load servicemenus from new install location
     const QStringList paths =
         QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kio/servicemenus"), QStandardPaths::LocateDirectory);
-    filePaths << KFileUtils::findAllUniqueFiles(paths, QStringList(QStringLiteral("*.desktop")));
+    const QStringList fromDisk = KFileUtils::findAllUniqueFiles(paths, QStringList(QStringLiteral("*.desktop")));
+    for (const QString &fileFromDisk : fromDisk) {
+        if (auto [_, inserted] = uniqueFileNames.insert(fileFromDisk.split(QLatin1Char('/')).last()); inserted) {
+            filePaths << fileFromDisk;
+        }
+    }
     return filePaths;
 }
 
