@@ -1,37 +1,25 @@
 /*
- *  This file is part of the KDE libraries
- *  Copyright (C) 2000-2001,2003,2010 Dawit Alemayehu <adawit at kde.org>
- *
- *  Original author
- *  Copyright (C) 2000 Yves Arrouye <yves@realnames.com>
- *
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- **/
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000-2001, 2003, 2010 Dawit Alemayehu <adawit at kde.org>
+
+    Original author
+    SPDX-FileCopyrightText: 2000 Yves Arrouye <yves@realnames.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef KURIFILTER_H
 #define KURIFILTER_H
 
 #include "kiowidgets_export.h"
 
-#include <QtCore/QObject>
-#include <QtCore/QHash>
-#include <QtCore/QPair>
-#include <QtCore/QStringList>
-#include <QtCore/QUrl>
+#include <QHash>
+#include <QObject>
+#include <QPair>
+#include <QStringList>
+#include <QUrl>
+
+#include <memory>
 
 #ifdef Q_OS_WIN
 #undef ERROR
@@ -43,6 +31,8 @@ class KCModule;
 class QHostInfo;
 
 /**
+ * @class KUriFilterSearchProvider kurifilter.h <KUriFilter>
+ *
  * Class that holds information about a search provider.
  *
  * @since 4.6
@@ -117,54 +107,49 @@ protected:
 private:
     friend class KUriFilterPlugin;
     class KUriFilterSearchProviderPrivate;
-    KUriFilterSearchProviderPrivate *const d;
+    std::unique_ptr<KUriFilterSearchProviderPrivate> const d;
 };
 
 /**
-* This class is a basic messaging class used to exchange filtering information
-* between the filter plugins and the application requesting the filtering
-* service.
-*
-* Use this object if you require a more detailed information about the URI you
-* want to filter. Any application can create an instance of this class and send
-* it to KUriFilter to have the plugins fill out all possible information about
-* the URI.
-*
-* On successful filtering you can use @ref uriType() to determine what type
-* of resource the request was filtered into. See @ref KUriFilter::UriTypes for
-* details. If an error is encountered, then @ref KUriFilter::Error is returned.
-* You can use @ref errorMsg to obtain the error information.
-*
-* The functions in this class are not reentrant.
-*
-* \b Example
-*
-* Here is a basic example of how this class is used with @ref KUriFilter:
-* \code
-*   KUriFilterData filterData (QLatin1String("kde.org"));
-*   bool filtered = KUriFilter::self()->filterUri(filterData);
-* \endcode
-*
-* If you are only interested in getting the list of preferred search providers,
-* then you can do the following (requires KDE >= 4.5):
-*
-* \code
-* KDE 4.5:
-* KUriFilterData data;
-* data.setData("<text-to-search-for>");
-* data.setAlternateDefaultSearchProvider("google");
-* bool filtered = KUriFilter::self()->filterUri(data, "kuriikwsfilter");
-*
-* KDE >= 4.6:
-* KUriFilterData data;
-* data.setData("<text-to-search-for>");
-* data.setSearchFilteringOption(KUriFilterData::RetrievePreferredSearchProvidersOnly);
-* bool filtered = KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter);
-* \endcode
-*
-* @short A class for exchanging filtering information.
-* @author Dawit Alemayehu <adawit at kde.org>
-*/
+ * @class KUriFilterData kurifilter.h <KUriFilter>
+ *
+ * This class is a basic messaging class used to exchange filtering information
+ * between the filter plugins and the application requesting the filtering
+ * service.
+ *
+ * Use this object if you require a more detailed information about the URI you
+ * want to filter. Any application can create an instance of this class and send
+ * it to KUriFilter to have the plugins fill out all possible information about
+ * the URI.
+ *
+ * On successful filtering you can use @ref uriType() to determine what type
+ * of resource the request was filtered into. See @ref KUriFilter::UriTypes for
+ * details. If an error is encountered, then @ref KUriFilter::Error is returned.
+ * You can use @ref errorMsg to obtain the error information.
+ *
+ * The functions in this class are not reentrant.
+ *
+ * \b Example
+ *
+ * Here is a basic example of how this class is used with @ref KUriFilter:
+ * \code
+ *   KUriFilterData filterData (QLatin1String("kde.org"));
+ *   bool filtered = KUriFilter::self()->filterUri(filterData);
+ * \endcode
+ *
+ * If you are only interested in getting the list of preferred search providers,
+ * then you can do the following:
+ *
+ * \code
+ * KUriFilterData data;
+ * data.setData("<text-to-search-for>");
+ * data.setSearchFilteringOption(KUriFilterData::RetrievePreferredSearchProvidersOnly);
+ * bool filtered = KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter);
+ * \endcode
+ *
+ * @short A class for exchanging filtering information.
+ * @author Dawit Alemayehu <adawit at kde.org>
+ */
 
 class KIOWIDGETS_EXPORT KUriFilterData
 {
@@ -188,34 +173,38 @@ public:
     enum UriTypes { NetProtocol = 0, LocalFile, LocalDir, Executable, Help, Shell, Blocked, Error, Unknown };
 
     /**
-      * This enum describes the search filtering options to be used.
-      *
-      * @li SearchFilterOptionNone
-      *     No search filter options are set and normal filtering is performed
-      *     on the input data.
-      * @li RetrieveSearchProvidersOnly
-      *     If set, the list of all available search providers are returned without
-      *     any input filtering. This flag only applies when used in conjunction
-      *     with the @ref KUriFilter::NormalTextFilter flag.
-      * @li RetrievePreferredSearchProvidersOnly
-      *     If set, the list of preferred search providers are returned without
-      *     any input filtering. This flag only applies when used in conjunction
-      *     with the @ref KUriFilter::NormalTextFilter flag.
-      * @li RetrieveAvailableSearchProvidersOnly
-      *     Same as doing RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly,
-      *     where all available search providers are returned if no preferred ones
-      *     ones are available. No input filtering will be performed.
-      *
-      * @see setSearchFilteringOptions
-      * @see KUriFilter::filterSearchUri
-      * @since 4.6
-      */
+     * This enum describes the search filtering options to be used.
+     *
+     * @li SearchFilterOptionNone
+     *     No search filter options are set and normal filtering is performed
+     *     on the input data.
+     * @li RetrieveSearchProvidersOnly
+     *     If set, the list of all available search providers are returned without
+     *     any input filtering. This flag only applies when used in conjunction
+     *     with the @ref KUriFilter::NormalTextFilter flag.
+     * @li RetrievePreferredSearchProvidersOnly
+     *     If set, the list of preferred search providers are returned without
+     *     any input filtering. This flag only applies when used in conjunction
+     *     with the @ref KUriFilter::NormalTextFilter flag.
+     * @li RetrieveAvailableSearchProvidersOnly
+     *     Same as doing RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly,
+     *     where all available search providers are returned if no preferred ones
+     *     ones are available. No input filtering will be performed.
+     *
+     * @see setSearchFilteringOptions
+     * @see KUriFilter::filterSearchUri
+     * @see SearchFilterOptions
+     * @since 4.6
+     */
     enum SearchFilterOption {
         SearchFilterOptionNone = 0x0,
         RetrieveSearchProvidersOnly = 0x01,
         RetrievePreferredSearchProvidersOnly = 0x02,
-        RetrieveAvailableSearchProvidersOnly = (RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly)
+        RetrieveAvailableSearchProvidersOnly = (RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly),
     };
+    /**
+     * Stores a combination of #SearchFilterOption values.
+     */
     Q_DECLARE_FLAGS(SearchFilterOptions, SearchFilterOption)
 
     /**
@@ -267,10 +256,10 @@ public:
      * Returns an error message.
      *
      * This functions returns the error message set by the plugin whenever the
-     * uri type is set to KUriFilterData::ERROR. Otherwise, it returns a NULL
+     * uri type is set to KUriFilterData::ERROR. Otherwise, it returns a nullptr
      * string.
      *
-     * @return the error message or a NULL when there is none.
+     * @return the error message or a nullptr when there is none.
      */
     QString errorMsg() const;
 
@@ -535,7 +524,7 @@ public:
      * a QUrl, make sure you set the argument for this function to the result
      * of calling path () instead of url ().
      *
-     * @param abs_path  the abolute path to the local resource.
+     * @param abs_path  the absolute path to the local resource.
      *
      * @return true if absolute path is successfully set. Otherwise, false.
      */
@@ -587,24 +576,24 @@ public:
     void setDefaultUrlScheme(const QString &);
 
     /**
-      * Sets the options used by search filter plugins to filter requests.
-      *
-      * The default search filter option is @ref SearchFilterOptionNone. See
-      * @ref SearchFilterOption for the description of the other flags.
-      *
-      * It is important to note that the options set through this function can
-      * prevent any filtering from being performed by search filter plugins.
-      * As such, @ref uriTypes can return KUriFilterData::Unknown and @ref uri
-      * can return an invalid url eventhough the filtering request returned
-      * a successful response.
-      *
-      * @see searchFilteringOptions
-      * @since 4.6
-      */
+     * Sets the options used by search filter plugins to filter requests.
+     *
+     * The default search filter option is @ref SearchFilterOptionNone. See
+     * @ref SearchFilterOption for the description of the other flags.
+     *
+     * It is important to note that the options set through this function can
+     * prevent any filtering from being performed by search filter plugins.
+     * As such, @ref uriTypes can return KUriFilterData::Unknown and @ref uri
+     * can return an invalid url even though the filtering request returned
+     * a successful response.
+     *
+     * @see searchFilteringOptions
+     * @since 4.6
+     */
     void setSearchFilteringOptions(SearchFilterOptions options);
 
     /**
-     * Overloaded assigenment operator.
+     * Overloaded assignment operator.
      *
      * This function allows you to easily assign a QUrl
      * to a KUriFilterData object.
@@ -614,7 +603,7 @@ public:
     KUriFilterData &operator=(const QUrl &url);
 
     /**
-     * Overloaded assigenment operator.
+     * Overloaded assignment operator.
      *
      * This function allows you to easily assign a QString to a KUriFilterData
      * object.
@@ -625,10 +614,12 @@ public:
 
 private:
     friend class KUriFilterPlugin;
-    KUriFilterDataPrivate *const d;
+    std::unique_ptr<KUriFilterDataPrivate> d;
 };
 
 /**
+ * @class KUriFilterPlugin kurifilter.h <KUriFilter>
+ *
  * Base class for URI filter plugins.
  *
  * This class applies a single filter to a URI. All plugins designed to provide
@@ -640,29 +631,32 @@ private:
  *
  * @short Abstract class for URI filter plugins.
  */
-class KIOWIDGETS_EXPORT KUriFilterPlugin : public QObject
+class KIOWIDGETS_EXPORT KUriFilterPlugin : public QObject // TODO KF6: move to separate header (maybe we don't need to install it then)
 {
     Q_OBJECT
 
 public:
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 6)
     /**
      * List for holding the following search provider information:
      * ([search provider name], [search query, search query icon name])
      *
      * @since 4.5
-     * @deprecated Use @ref KUriFilterSearchProvider instead. See @ref setSearchProviders;
+     * @deprecated Since 4.6, use @ref KUriFilterSearchProvider instead. See @ref setSearchProviders;
      */
-#ifndef KDE_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED typedef QHash<QString, QPair<QString, QString> > ProviderInfoList;
+    KIOWIDGETS_DEPRECATED_VERSION(4, 6, "Use KUriFilterSearchProvider")
+    typedef QHash<QString, QPair<QString, QString>> ProviderInfoList;
 #endif
 
     /**
      * Constructs a filter plugin with a given name
      *
-     * @param parent the parent object, or 0 for no parent
+     * @param parent the parent object, or @c nullptr for no parent
      * @param name the name of the plugin, mandatory
      */
-    explicit KUriFilterPlugin(const QString &name, QObject *parent = 0);
+    explicit KUriFilterPlugin(const QString &name, QObject *parent = nullptr);
+
+    // KF6 TODO ~KUriFilterPlugin();
 
     /**
      * Filters a URI.
@@ -678,16 +672,16 @@ public:
      * It is the responsibility of the caller to delete the module once it is
      * not needed anymore.
      *
-     * @return A configuration module, 0 if the filter isn't configurable.
+     * @return A configuration module, or @c nullptr if the filter isn't configurable.
      */
-    virtual KCModule *configModule(QWidget *, const char *) const;
+    virtual KCModule *configModule(QWidget *, const char *) const; // TODO KF6: unused, remove
 
     /**
      * Returns the name of the configuration module for the filter.
      *
      * @return the name of a configuration module or QString() if none.
      */
-    virtual QString configName() const;
+    virtual QString configName() const; // TODO KF6: unused, remove
 
 protected:
     /**
@@ -707,7 +701,7 @@ protected:
 
     /**
      * Sets the arguments and options string in @p data to @p args if any were
-     * found during filterting.
+     * found during filtering.
      */
     void setArguments(KUriFilterData &data, const QString &args) const;
 
@@ -717,8 +711,7 @@ protected:
      *
      * @since 4.5
      */
-    void setSearchProvider(KUriFilterData &data, const QString &provider,
-                           const QString &term, const QChar &separator) const;
+    void setSearchProvider(KUriFilterData &data, const QString &provider, const QString &term, const QChar &separator) const;
 
     /**
      * Sets the information about the search @p providers in @p data.
@@ -757,6 +750,8 @@ private:
 };
 
 /**
+ * @class KUriFilter kurifilter.h <KUriFilter>
+ *
  * KUriFilter applies a number of filters to a URI and returns a filtered version if any
  * filter matches.
  * A simple example is "kde.org" to "http://www.kde.org", which is commonplace in web browsers.
@@ -795,7 +790,7 @@ private:
  * "http://kde.org".
  *
  * You can also restrict the filters to be used by supplying the name of the
- * filters you want to use. By defualt all available filters are used.
+ * filters you want to use. By default all available filters are used.
  *
  * To use specific filters, add the names of the filters you want to use to a
  * QStringList and invoke the appropriate filtering function.
@@ -860,15 +855,20 @@ class KIOWIDGETS_EXPORT KUriFilter
 {
 public:
     /**
-    * This enum describes the types of search plugin filters available.
-    *
-    * @li NormalTextFilter      The plugin used to filter normal text, e.g. "some term to search".
-    * @li WebShortcutFilter     The plugin used to filter web shortcuts, e.g. gg:KDE.
-    */
+     * This enum describes the types of search plugin filters available.
+     *
+     * @li NormalTextFilter      The plugin used to filter normal text, e.g. "some term to search".
+     * @li WebShortcutFilter     The plugin used to filter web shortcuts, e.g. gg:KDE.
+     *
+     * @see SearchFilterTypes
+     */
     enum SearchFilterType {
         NormalTextFilter = 0x01,
-        WebShortcutFilter = 0x02
+        WebShortcutFilter = 0x02,
     };
+    /**
+     * Stores a combination of #SearchFilterType values.
+     */
     Q_DECLARE_FLAGS(SearchFilterTypes, SearchFilterType)
 
     /**
@@ -946,14 +946,15 @@ public:
      */
     QString filteredUri(const QString &uri, const QStringList &filters = QStringList());
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 6)
     /**
      * See @ref filterSearchUri(KUriFilterData&, SearchFilterTypes)
      *
      * @since 4.5
-     * @deprecated Use filterSearchUri(KUriFilterData&, SearchFilterTypes) instead.
+     * @deprecated Since 4.6, use filterSearchUri(KUriFilterData&, SearchFilterTypes) instead.
      */
-#ifndef KDE_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED bool filterSearchUri(KUriFilterData &data);
+    KIOWIDGETS_DEPRECATED_VERSION(4, 6, "Use KUriFilter::filterSearchUri(KUriFilterData &, SearchFilterTypes)")
+    bool filterSearchUri(KUriFilterData &data);
 #endif
 
     /**
@@ -982,7 +983,6 @@ public:
     QStringList pluginNames() const;
 
 protected:
-
     /**
      * Constructor.
      *
@@ -999,7 +999,7 @@ protected:
     void loadPlugins();
 
 private:
-    KUriFilterPrivate *const d;
+    std::unique_ptr<KUriFilterPrivate> const d;
     friend class KUriFilterSingleton;
 };
 
